@@ -4,7 +4,6 @@
 //! converts them to [rust-bitcoin](https://github.com/rust-bitcoin/rust-bitcoin) objects.
 //! For details, please see [Context](./struct.Context.html).
 
-use std::str::FromStr;
 #[cfg(feature="softforks")]
 use std::collections::HashMap;
 use serde::Deserialize;
@@ -15,7 +14,7 @@ use bitcoin::consensus::Decodable;
 
 pub const DEFAULT_ENDPOINT: &str = "http://localhost:8332/rest/";
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 pub struct Softfork {
     #[serde(rename="type")]
     pub type_: String,
@@ -24,7 +23,7 @@ pub struct Softfork {
     pub height: u32,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 pub struct ChainInfo {
     pub chain: String,
     pub blocks: u32,
@@ -42,7 +41,7 @@ pub struct ChainInfo {
     pub warnings: String,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct ScriptPubKey {
     pub asm: String,
@@ -55,7 +54,7 @@ pub struct ScriptPubKey {
     pub addresses: Vec<String>,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Utxo {
     pub height: u32,
@@ -63,7 +62,7 @@ pub struct Utxo {
     pub script_pub_key: ScriptPubKey,
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct UtxoData {
     pub chain_height: u32,
@@ -72,6 +71,24 @@ pub struct UtxoData {
     pub utxos: Vec<Utxo>,
 }
 
+#[derive(Debug)]
+pub enum Error {
+    Reqwest(reqwest::Error),
+    BitcoinEncodeError(bitcoin::consensus::encode::Error)
+}
+
+impl From<reqwest::Error> for Error {
+    fn from(err: reqwest::Error) -> Self {
+        Self::Reqwest(err)
+    }
+}
+
+impl From<bitcoin::consensus::encode::Error> for Error {
+    fn from(err: bitcoin::consensus::encode::Error) -> Self {
+        Self::BitcoinEncodeError(err)
+    }
+}
+
 /// `bitcoin_rest` context.
 #[derive(Debug, Clone)]
 pub struct Context {
@@ -118,48 +135,49 @@ impl Context {
         Ok(result)
     }
     /// Call the [/tx](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#transactions) endpoint.
-    pub async fn tx(&self, txhash: &Txid) -> Result<Transaction, reqwest::Error> {
+    pub async fn tx(&self, txhash: &Txid) -> Result<Transaction, Error> {
         let path = String::from("tx/") + &txhash.to_string();
         let result = self.call_bin(&path).await?;
-        Ok(Transaction::consensus_decode(result.as_ref()).unwrap())
+        Ok(Transaction::consensus_decode(result.as_ref())?)
     }
     /// Call the [/block](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blocks) endpoint.
-    pub async fn block(&self, blockhash: &BlockHash) -> Result<Block, reqwest::Error> {
+    pub async fn block(&self, blockhash: &BlockHash) -> Result<Block, Error> {
         let path = String::from("block/") + &blockhash.to_string();
         let result = self.call_bin(&path).await?;
-        Ok(Block::consensus_decode(result.as_ref()).unwrap())
+        Ok(Block::consensus_decode(result.as_ref())?)
     }
     /// Call the [/block/notxdetails](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blocks) endpoint.
-    pub async fn block_notxdetails(&self, blockhash: &BlockHash) -> Result<BlockHeader, reqwest::Error> {
+    pub async fn block_notxdetails(&self, blockhash: &BlockHash) -> Result<BlockHeader, Error> {
         let path = String::from("block/notxdetails/") + &blockhash.to_string();
         let result = self.call_bin(&path).await?;
-        Ok(BlockHeader::consensus_decode(result.as_ref()).unwrap())
+        Ok(BlockHeader::consensus_decode(result.as_ref())?)
     }
     /// Call the [/headers](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blockheaders) endpoint.
-    pub async fn headers(&self, count: u32, blockhash: &BlockHash) -> Result<Vec<BlockHeader>, reqwest::Error> {
+    pub async fn headers(&self, count: u32, blockhash: &BlockHash) -> Result<Vec<BlockHeader>, Error> {
         let path = String::from("headers/") + &count.to_string() + "/" + &blockhash.to_string();
         let result = self.call_bin(&path).await?;
         let mut ret = Vec::new();
-        for i in 0..count {
-            let begin = (i as usize) * 80usize;
-            let end = ((i + 1) as usize) * 80usize;
-            ret.push(BlockHeader::consensus_decode(result.slice(begin .. end).as_ref()).unwrap());
+        const BLOCK_HEADER_SIZE: usize = 80usize;
+        let mut offset = 0;
+        while offset < result.len() {
+            ret.push(BlockHeader::consensus_decode(result[offset..(offset+BLOCK_HEADER_SIZE)].as_ref())?);
+            offset += BLOCK_HEADER_SIZE;
         }
         Ok(ret)
     }
     /// Call the [/blockhashbyheight](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blockhash-by-height) endpoint.
-    pub async fn blockhashbyheight(&self, height: u32) -> Result<BlockHash, reqwest::Error> {
+    pub async fn blockhashbyheight(&self, height: u32) -> Result<BlockHash, Error> {
         let path = String::from("blockhashbyheight/") + &height.to_string();
-        let result = self.call_hex(&path).await?;
-        Ok(BlockHash::from_str(&result).unwrap())
+        let result = self.call_bin(&path).await?;
+        Ok(BlockHash::consensus_decode(result.as_ref())?)
     }
     /// Call the [/chaininfo](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#chaininfo) endpoint.
-    pub async fn chaininfo(&self) -> Result<ChainInfo, reqwest::Error> {
+    pub async fn chaininfo(&self) -> Result<ChainInfo, Error> {
         let result: ChainInfo = self.call_json("chaininfo").await?;
         Ok(result)
     }
     /// Call the [/getutxos](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#query-utxo-set) endpoint.
-    pub async fn getutxos(&self, checkmempool: bool, txids: &Vec<Txid>) -> Result<UtxoData, reqwest::Error> {
+    pub async fn getutxos(&self, checkmempool: bool, txids: &Vec<Txid>) -> Result<UtxoData, Error> {
         let mut url = String::from("getutxos/");
         if checkmempool {
             url += "checkmempool/"
@@ -172,9 +190,9 @@ impl Context {
     }
 }
 
-
 #[cfg(test)]
 mod tests {
+    use std::str::FromStr;
     use super::*;
     struct Fixture {
         rest_env_name: &'static str,
